@@ -353,7 +353,7 @@ void burn_disc_erase(struct burn_drive *drive, int fast)
 	/* a ssert(!SCAN_GOING()); */
 	/* a ssert(!find_worker(drive)); */
 
-	if((drive == NULL)) {
+	if(drive == NULL) {
 		libdax_msgs_submit(libdax_messenger, -1,
 			0x00020104,
 			LIBDAX_MSGS_SEV_SORRY, LIBDAX_MSGS_PRIO_HIGH,
@@ -395,11 +395,16 @@ void burn_disc_erase(struct burn_drive *drive, int fast)
 	    ||
 	    (drive->drive_role != 1 && drive->drive_role != 5)
 	   ) {
+		char msg[160];
+
+		sprintf(msg, "Drive and media state unsuitable for blanking. (role= %d , profile= 0x%x , status= %d)",
+			drive->drive_role,
+			(unsigned int) drive->current_profile,
+			drive->status);
 		libdax_msgs_submit(libdax_messenger, drive->global_index,
 			0x00020130,
 			LIBDAX_MSGS_SEV_SORRY, LIBDAX_MSGS_PRIO_HIGH,
-			"Drive and media state unsuitable for blanking",
-			0, 0);
+			msg, 0, 0);
 		return;
 	}
 
@@ -638,8 +643,7 @@ void burn_disc_write(struct burn_write_opts *opts, struct burn_disc *disc)
 	if (d->drive_role == 1) {
 		mvalid = 0;
 		if (d->mdata != NULL)
-			if (d->mdata->valid > 0)
-				mvalid = 1;
+			mvalid = 1;
 		if (!mvalid) {
 			libdax_msgs_submit(libdax_messenger,
 				d->global_index, 0x00020113,
