@@ -1,7 +1,7 @@
 /* -*- indent-tabs-mode: t; tab-width: 8; c-basic-offset: 8; -*- */
 
 /* Copyright (c) 2004 - 2006 Derek Foreman, Ben Jansens
-   Copyright (c) 2006 - 2013 Thomas Schmitt <scdbackup@gmx.net>
+   Copyright (c) 2006 - 2014 Thomas Schmitt <scdbackup@gmx.net>
    Provided under GPL version 2 or later.
 */
 
@@ -80,6 +80,7 @@ int burn_setup_drive(struct burn_drive *d, char *fname)
 	d->do_stream_recording = 0;
         d->stream_recording_start= 0;
 	d->role_5_nwa = 0;
+	d->features = NULL;
 	return 1;
 }
 
@@ -104,6 +105,7 @@ void burn_drive_free_subs(struct burn_drive *d)
 	if (d->stdio_fd >= 0)
 		close (d->stdio_fd);
 	d->stdio_fd = -1;
+	burn_feature_descr_free(&(d->features), 0);
 	sg_dispose_drive(d, 0);
 }
 
@@ -261,8 +263,9 @@ int burn_drive_inquire_media(struct burn_drive *d)
 
         /* ts A71128 : run read_disc_info() for any recognizeable profile */
 	if (d->current_profile > 0 || d->current_is_guessed_profile ||
-	    d->mdata->cdr_write || d->mdata->cdrw_write ||
-	    d->mdata->dvdr_write || d->mdata->dvdram_write) {
+	    (d->mdata->p2a_valid > 0 &&
+		(d->mdata->cdr_write || d->mdata->cdrw_write ||
+		 d->mdata->dvdr_write || d->mdata->dvdram_write)) ) {
 
 #define Libburn_knows_correct_state_after_loaD 1
 #ifdef Libburn_knows_correct_state_after_loaD
@@ -424,6 +427,8 @@ fprintf(stderr, "LIBBURN_DEBUG: burn_drive__is_rdwr: getfl_ret = %lX , O_RDWR =
 }
 
 
+/* flag bit0= ( not needed yet: grab even if it is already grabbed )
+*/
 int burn_drive_grab_stdio(struct burn_drive *d, int flag)
 {
 	int stat_ret = -1, is_rdwr, ret;
@@ -537,6 +542,10 @@ int burn_drive_grab(struct burn_drive *d, int le)
 		{ret = 0; goto ex;}
 
 ex:;
+	if (d->cancel || burn_is_aborting(0)) {
+	        d->unlock(d);
+		d->release(d);
+	}
 	d->silent_on_scsi_error = sose;
 	d->busy = BURN_DRIVE_IDLE;
 	burn_grab_restore_sig_action(signal_action_mem, 0);
@@ -570,6 +579,7 @@ struct burn_drive *burn_drive_register(struct burn_drive *d)
 	d->toc_entry = NULL;
 	d->disc = NULL;
 	d->erasable = 0;
+	d->write_opts = NULL;
 
 #ifdef Libburn_ticket_62_re_register_is_possiblE
 	/* ts A60904 : ticket 62, contribution by elmom */
@@ -638,7 +648,7 @@ struct burn_drive *burn_drive_finish_enum(struct burn_drive *d)
 	        t->released = 1;
 	} else {
 		/* ts A90602 */
-		d->mdata->valid = -1;
+		d->mdata->p2a_valid = -1;
                 sprintf(msg, "Unable to grab scanned drive %s", d->devname);
                 libdax_msgs_submit(libdax_messenger, d->global_index,
                                 0x0002016f, LIBDAX_MSGS_SEV_DEBUG,
@@ -675,6 +685,10 @@ int burn_drive_mark_unready(struct burn_drive *d, int flag)
 		free(d->toc_entry);
 	d->toc_entry = NULL;
 	d->toc_entries = 0;
+	if (d->write_opts != NULL) {
+		burn_write_opts_free(d->write_opts);
+		d->write_opts = NULL;
+	}
 	if (d->disc != NULL) {
 		burn_disc_free(d->disc);
 		d->disc = NULL;
@@ -844,7 +858,7 @@ void burn_wait_all(void)
 
 void burn_disc_erase_sync(struct burn_drive *d, int fast)
 {
-	int ret;
+	int ret, was_error = 0;
 
 	if (d->drive_role == 5) { /* Random access write-only drive */
 		ret = truncate(d->devname, (off_t) 0);
@@ -898,12 +912,16 @@ void burn_disc_erase_sync(struct burn_drive *d, int fast)
 		ret = d->get_erase_progress(d);
 		if (ret == -2 || ret > 0)
 	break;
+		if (ret == -3)
+			was_error = 1;
 		sleep(1);
 	}
 	while (1) {
 		ret = d->get_erase_progress(d);
 		if(ret == -2)
 	break;
+		if (ret == -3)
+			was_error = 1;
 		if (ret >= 0)
 			d->progress.sector = ret;
 		sleep(1);
@@ -918,6 +936,8 @@ void burn_disc_erase_sync(struct burn_drive *d, int fast)
 	if (d->drive_role == 1)
 		burn_drive_inquire_media(d);
 	d->busy = BURN_DRIVE_IDLE;
+	if (was_error)
+		d->cancel = 1;
 }
 
 /*
@@ -927,6 +947,7 @@ void burn_disc_erase_sync(struct burn_drive *d, int fast)
 void burn_disc_format_sync(struct burn_drive *d, off_t size, int flag)
 {
 	int ret, buf_secs, err, i, stages = 1, pbase, pfill, pseudo_sector;
+	int was_error = 0;
 	off_t num_bufs;
 	char msg[80];
 	struct buffer *buf = NULL, *buf_mem = d->buffer;
@@ -971,12 +992,16 @@ void burn_disc_format_sync(struct burn_drive *d, off_t size, int flag)
 		ret = d->get_erase_progress(d);
 		if (ret == -2 || ret > 0)
 	break;
+		if (ret == -3)
+			was_error = 1;
 		sleep(1);
 	}
 	while (1) {
 		pseudo_sector = d->get_erase_progress(d);
 		if(pseudo_sector == -2)
 	break;
+		if (pseudo_sector == -3)
+			was_error = 1;
 		if (pseudo_sector >= 0)
 			d->progress.sector = pseudo_sector / stages;
 		sleep(1);
@@ -1040,6 +1065,8 @@ ex:;
 	d->progress.sector = 0x10000;
 	d->busy = BURN_DRIVE_IDLE;
 	d->buffer = buf_mem;
+	if (was_error)
+		d->cancel = 1;
 	BURN_FREE_MEM(buf);
 }
 
@@ -1194,6 +1221,8 @@ static void strip_spaces(char *str)
 static int drive_getcaps(struct burn_drive *d, struct burn_drive_info *out)
 {
 	struct burn_scsi_inquiry_data *id;
+	int i, profile;
+	struct burn_feature_descr *feat;
 
 	/* ts A61007 : now prevented in enumerate_common() */
 #if 0
@@ -1201,7 +1230,7 @@ static int drive_getcaps(struct burn_drive *d, struct burn_drive_info *out)
 	a ssert(d->mdata);
 #endif
 
-	if(d->idata->valid <= 0 || d->mdata->valid <= 0)
+	if(d->idata->valid <= 0)
 		return 0;
 
 	id = (struct burn_scsi_inquiry_data *)d->idata;
@@ -1214,18 +1243,48 @@ static int drive_getcaps(struct burn_drive *d, struct burn_drive_info *out)
 	strip_spaces(out->revision);
 	strncpy(out->location, d->devname, 16);
 	out->location[16] = '\0';
-	out->buffer_size = d->mdata->buffer_size;
-	out->read_dvdram = !!d->mdata->dvdram_read;
-	out->read_dvdr = !!d->mdata->dvdr_read;
-	out->read_dvdrom = !!d->mdata->dvdrom_read;
-	out->read_cdr = !!d->mdata->cdr_read;
-	out->read_cdrw = !!d->mdata->cdrw_read;
-	out->write_dvdram = !!d->mdata->dvdram_write;
-	out->write_dvdr = !!d->mdata->dvdr_write;
-	out->write_cdr = !!d->mdata->cdr_write;
-	out->write_cdrw = !!d->mdata->cdrw_write;
-	out->write_simulate = !!d->mdata->simulate;
-	out->c2_errors = !!d->mdata->c2_pointers;
+
+	if (d->mdata->p2a_valid > 0) {
+		out->buffer_size = d->mdata->buffer_size;
+		out->read_dvdram = !!d->mdata->dvdram_read;
+		out->read_dvdr = !!d->mdata->dvdr_read;
+		out->read_dvdrom = !!d->mdata->dvdrom_read;
+		out->read_cdr = !!d->mdata->cdr_read;
+		out->read_cdrw = !!d->mdata->cdrw_read;
+		out->write_dvdram = !!d->mdata->dvdram_write;
+		out->write_dvdr = !!d->mdata->dvdr_write;
+		out->write_cdr = !!d->mdata->cdr_write;
+		out->write_cdrw = !!d->mdata->cdrw_write;
+		out->write_simulate = !!d->mdata->simulate;
+		out->c2_errors = !!d->mdata->c2_pointers;
+	} else {
+		out->buffer_size = out->read_dvdram = out->read_dvdr = 0;
+		out->read_dvdrom = out->read_cdr = out->read_cdrw = 0;
+		out->write_dvdram = out->write_dvdr = out->write_cdr = 0;
+		out->write_cdrw = out->write_simulate = out->c2_errors = 0;
+		for (i = 0; i < d->num_profiles; i++) {
+			profile = (d->all_profiles[i * 4] << 8) |
+						d->all_profiles[i * 4 + 1];
+			if (profile == 0x09)
+				out->write_cdr = out->read_cdr = 1;
+			else if (profile == 0x0a)
+				out->write_cdrw = out->read_cdrw = 1;
+			else if (profile == 0x10)
+				out->read_dvdrom = 1;
+			else if (profile == 0x11)
+				out->write_dvdr = out->read_dvdr = 1;
+			else if (profile == 0x12)
+				out->write_dvdram = out->read_dvdram = 1;
+		}
+
+		/* Test Write bit of CD TAO, CD Mastering, DVD-R/-RW Write */
+		for (i = 0x002D; i <= 0x002F; i++)
+			if (burn_drive_has_feature(d, i, &feat, 0))
+				if (feat->data_lenght > 0)
+					out->write_simulate |=
+							 !!(feat->data[0] & 4);
+	}
+
 	out->drive = d;
 
 #ifdef Libburn_dummy_probe_write_modeS
@@ -1544,23 +1603,17 @@ void burn_sectors_to_msf(int sectors, int *m, int *s, int *f)
 
 int burn_drive_get_read_speed(struct burn_drive *d)
 {
-	if(d->mdata->valid <= 0)
-		return 0;
 	return d->mdata->max_read_speed;
 }
 
 int burn_drive_get_write_speed(struct burn_drive *d)
 {
-	if(d->mdata->valid <= 0)
-		return 0;
 	return d->mdata->max_write_speed;
 }
 
 /* ts A61021 : New API function */
 int burn_drive_get_min_write_speed(struct burn_drive *d)
 {
-	if(d->mdata->valid <= 0)
-		return 0;
 	return d->mdata->min_write_speed;
 }
 
@@ -1852,8 +1905,10 @@ int burn_drive_scan_and_grab(struct burn_drive_info *drive_infos[], char* adr,
 */
 
 	ret = burn_drive_grab(drive_infos[0]->drive, load);
-	if (ret != 1)
+	if (ret != 1) {
+		burn_drive_forget(drive_infos[0]->drive, 0);
 		return -1;
+	}
 	return 1;
 }
 
@@ -2476,6 +2531,13 @@ int burn_disc_pretend_full(struct burn_drive *d)
 	return 1;
 }
 
+/* ts B31110 API function */
+int burn_disc_pretend_full_uncond(struct burn_drive *d)
+{
+	d->status = BURN_DISC_FULL;
+	return 1;
+}
+
 /* ts A61021: new API function */
 int burn_disc_read_atip(struct burn_drive *d)
 {
@@ -2490,7 +2552,8 @@ int burn_disc_read_atip(struct burn_drive *d)
 	if(d->drive_role != 1)
 		return 0;
 	if ((d->current_profile == -1 || d->current_is_cd_profile)
-	    && (d->mdata->cdrw_write || d->current_profile != 0x08)) {
+	    && ((d->mdata->p2a_valid > 0 && d->mdata->cdrw_write) ||
+	        d->current_profile != 0x08)) {
 		d->read_atip(d);
 		/* >>> some control of success would be nice :) */
 	} else {
@@ -2719,8 +2782,6 @@ int burn_speed_descriptor_copy(struct burn_speed_descriptor *from,
 /* ts A61226 : free dynamically allocated sub data of struct scsi_mode_data */
 int burn_mdata_free_subs(struct scsi_mode_data *m)
 {
-	if(!m->valid)
-		return 0;
 	burn_speed_descriptor_destroy(&(m->speed_descriptors), 1);
 	return 1;
 }
@@ -2734,8 +2795,6 @@ int burn_drive_get_speedlist(struct burn_drive *d,
 	struct burn_speed_descriptor *sd, *csd = NULL;
 
 	(*speed_list) = NULL;
-	if(d->mdata->valid <= 0)
-		return 0;
 	for (sd = d->mdata->speed_descriptors; sd != NULL; sd = sd->next) {
 		ret = burn_speed_descriptor_new(&csd, NULL, csd, 0);
 		if (ret <= 0)
@@ -2759,8 +2818,6 @@ int burn_drive_get_best_speed(struct burn_drive *d, int speed_goal,
 	if (speed_goal < 0)
 		best_speed = 2000000000;
 	*best_descr = NULL;
-	if(d->mdata->valid <= 0)
-		return 0;
 	for (sd = d->mdata->speed_descriptors; sd != NULL; sd = sd->next) {
 		if (flag & 1)
 			speed = sd->read_speed;
@@ -2904,7 +2961,8 @@ int burn_disc_get_multi_caps(struct burn_drive *d, enum burn_write_types wt,
 			o->multi_session = o->multi_track = 0;
 		else if(wt == BURN_WRITE_NONE || wt == BURN_WRITE_SAO ||
 			wt == BURN_WRITE_TAO)
-			o->might_simulate = !!d->mdata->simulate;
+			o->might_simulate = !!(d->mdata->p2a_valid > 0 &&
+			                       d->mdata->simulate);
 	} else if (d->current_profile == 0x11 || d->current_profile == 0x14 ||
 			d->current_profile == 0x15) {
 		/* DVD-R , sequential DVD-RW , DVD-R/DL Sequential */
@@ -3389,3 +3447,80 @@ int burn_disc_get_leadin_text(struct burn_drive *d,
 	return ret;
 }
 
+
+/* ts B31023 API */
+/* Inquire for DVD-RW failure of TAO
+*/
+int burn_drive_was_feat21_failure(struct burn_drive *d)
+{
+	return !!d->was_feat21h_failure;
+}
+
+
+/* ts B40106 */
+int burn_feature_descr_new(struct burn_feature_descr **new,
+                           unsigned char *descr, int descr_len, int flag)
+{
+	struct burn_feature_descr *o;
+
+	*new = NULL;
+	if (descr_len < 4)
+		return 0;
+	(*new) = o = calloc(1, sizeof(struct burn_speed_descriptor));
+	if (o == NULL)
+		return -1;
+	o->feature_code = (descr[0] << 8) | descr[1];
+        o->flags = descr[2];
+        if (descr[3] > descr_len - 4)
+		o->data_lenght = 0;
+	else
+        	o->data_lenght = descr[3];
+        o->data = NULL;
+        o->next = NULL;
+        if (o->data_lenght > 0) {
+		o->data = calloc(1, o->data_lenght);
+		if (o->data == NULL) {
+			burn_feature_descr_free(new, 0);
+			return -1;
+		}
+		memcpy(o->data, descr + 4, o->data_lenght);
+	}
+	return 1;
+}
+
+
+/* ts B40106 */
+int burn_feature_descr_free(struct burn_feature_descr **descr, int flag)
+{
+	struct burn_feature_descr *o, *next;
+
+	if (*descr == NULL)
+		return 0;
+	for (o = *descr; o != NULL; o = next) {
+		next = o->next;
+		if (o->data != NULL)
+			free(o->data);
+		free((char *) o);
+	}
+	*descr = NULL;
+	return 1;
+}
+
+
+/* ts B40107 */
+int burn_drive_has_feature(struct burn_drive *d, int feature_code,
+                           struct burn_feature_descr **descr, int flag)
+{
+	struct burn_feature_descr *o;
+
+	for (o = d->features; o != NULL; o = o->next) {
+		if (o->feature_code == feature_code) {
+			if (descr != NULL)
+				*descr = o;
+			return 1;
+		}
+	}
+	return 0;
+}
+
+
