@@ -20,6 +20,7 @@
 
 #include <stdlib.h>
 #include <string.h>
+#include <stdio.h>
 
 #include "libdax_msgs.h"
 extern struct libdax_msgs *libdax_messenger;
@@ -43,7 +44,8 @@ struct burn_write_opts *burn_write_opts_new(struct burn_drive *drive)
 	opts->toc_entry = NULL;
 	opts->toc_entries = 0;
 	opts->simulate = 0;
-	opts->underrun_proof = drive->mdata->underrun_proof;
+	opts->underrun_proof = drive->mdata->p2a_valid > 0 &&
+	                       drive->mdata->underrun_proof;
 	opts->perform_opc = 1;
 	opts->obs = -1;
 
@@ -78,6 +80,25 @@ void burn_write_opts_free(struct burn_write_opts *opts)
 	free(opts);
 }
 
+int burn_write_opts_clone(struct burn_write_opts *from,
+                          struct burn_write_opts **to, int flag)
+{
+	if (*to != NULL)
+		burn_write_opts_free(*to);
+	if (from == NULL)
+		return 1;
+	*to = calloc(1, sizeof(struct burn_write_opts));
+	if (*to == NULL) {
+		libdax_msgs_submit(libdax_messenger, -1, 0x00000003,
+				LIBDAX_MSGS_SEV_FATAL, LIBDAX_MSGS_PRIO_HIGH,
+				"Out of virtual memory", 0, 0);
+		return -1;
+	}
+	memcpy(*to, from, sizeof(struct burn_write_opts));
+	(*to)->refcount= 1;
+	return 1;
+}
+
 struct burn_read_opts *burn_read_opts_new(struct burn_drive *drive)
 {
 	struct burn_read_opts *opts;
@@ -93,6 +114,7 @@ struct burn_read_opts *burn_read_opts_new(struct burn_drive *drive)
 	opts->report_recovered_errors = 0;
 	opts->transfer_damaged_blocks = 0;
 	opts->hardware_error_retries = 3;
+	opts->dap_bit = 0;
 
 	return opts;
 }
@@ -151,17 +173,6 @@ void burn_write_opts_set_format(struct burn_write_opts *opts, int format)
 
 int burn_write_opts_set_simulate(struct burn_write_opts *opts, int sim)
 {
-/* <<< ts A70529 :
-       One cannot predict the ability to simulate from page 05h
-       information alone. This check is now done later in 
-       function  burn_write_opts_auto_write_type().
-
-	if (opts->drive->mdata->simulate) {
-		opts->simulate = sim;
-		return 1;
-	}
-	return 0;
-*/
 	opts->simulate = !!sim;
 	return 1;
 }
@@ -169,9 +180,8 @@ int burn_write_opts_set_simulate(struct burn_write_opts *opts, int sim)
 int burn_write_opts_set_underrun_proof(struct burn_write_opts *opts,
 				       int underrun_proof)
 {
-	if (opts->drive->mdata->valid <= 0)
-		return 0;
-	if (opts->drive->mdata->underrun_proof) {
+	if (opts->drive->mdata->p2a_valid <= 0 ||
+	    opts->drive->mdata->underrun_proof) {
 		opts->underrun_proof = underrun_proof;
 		return 1;
 	}
@@ -203,6 +213,21 @@ void burn_write_opts_set_multi(struct burn_write_opts *opts, int multi)
 }
 
 
+/* ts B31024 */
+/* API */
+void burn_write_opts_set_fail21h_sev(struct burn_write_opts *opts,
+                                     char *severity)
+{
+	int ret, sevno;
+
+	ret = libdax_msgs__text_to_sev(severity, &sevno, 0);
+	if (ret <= 0)
+		opts->feat21h_fail_sev = 0;
+	else
+		opts->feat21h_fail_sev = sevno;
+}
+
+
 /* ts B11204 */
 /* @param flag bit0=do not verify checksums
                bit1= repair mismatching checksums
