@@ -1,7 +1,7 @@
 /* -*- indent-tabs-mode: t; tab-width: 8; c-basic-offset: 8; -*- */
 
 /* Copyright (c) 2004 - 2006 Derek Foreman, Ben Jansens
-   Copyright (c) 2006 - 2013 Thomas Schmitt <scdbackup@gmx.net>
+   Copyright (c) 2006 - 2014 Thomas Schmitt <scdbackup@gmx.net>
    Provided under GPL version 2 or later.
 */
 
@@ -436,7 +436,8 @@ static int sgio_test(int fd)
 	c_end_time = burn_get_time(0);
 
 	sgio_log_reply(s.cmdp, NO_TRANSFER, NULL, 0, NULL,
-	               s.sbp, s.sb_len_wr, c_end_time - c_start_time, 0);
+	               (unsigned char *) (s.sbp),
+	               s.sb_len_wr, c_end_time - c_start_time, 0);
 	return ret;
 }
 
@@ -483,7 +484,8 @@ static int sgio_inquiry_cd_drive(int fd, char *fname)
 	}
 
 	sgio_log_reply(s.cmdp, FROM_DRIVE, buf->data, s.dxfer_len, NULL,
-                       s.sbp, s.sb_len_wr, c_end_time - c_start_time, 0);
+	               (unsigned char *) (s.sbp),
+	               s.sb_len_wr, c_end_time - c_start_time, 0);
 
 	if (s.sb_len_wr > 0 || s.host_status != Libburn_sg_host_oK ||
 	    s.driver_status != Libburn_sg_driver_oK) {
@@ -492,7 +494,8 @@ static int sgio_inquiry_cd_drive(int fd, char *fname)
 			sprintf(msg + strlen(msg), " , sense data=");
 			msg_pt = msg + strlen(msg);
 			for (i = 0 ; i < s.sb_len_wr; i++)
-				sprintf(msg_pt + i * 3, " %2.2X", s.sbp[i]);
+				sprintf(msg_pt + i * 3, " %2.2X",
+				        ((unsigned char *) (s.sbp))[i]);
 		}
 		libdax_msgs_submit(libdax_messenger, -1, 0x00000002,
 			LIBDAX_MSGS_SEV_DEBUG, LIBDAX_MSGS_PRIO_HIGH,
@@ -1832,6 +1835,16 @@ try_open:;
 		d->fd = fd;
 		fcntl(fd, F_SETOWN, getpid());
 		d->released = 0;
+
+/* <<< Test for NetBSD */
+		ret = spc_confirm_cd_drive(d, 0);
+		if (ret <= 0) {
+			sg_close_drive(d);
+			d->fd = -1337;
+			return 0;
+		}
+/* <<< End Test for NetBSD */
+
 		return 1;
 	} else if (errno == EBUSY)
 		goto drive_is_in_use;
@@ -2166,6 +2179,24 @@ int sg_issue_command(struct burn_drive *d, struct command *c)
 
 		c->end_time = burn_get_time(0);
 
+/*
+ # de fine Libburn_ff_netbsd_mockuP
+ */
+#ifdef Libburn_ff_netbsd_mockuP
+		if (c->opcode[0] == 0x5a) {
+			sprintf(msg, "Libburn_ff_netbsd_mockuP : Emulating zero reply from SCSI command 5A");
+			if (burn_sg_log_scsi & 3)
+				scsi_log_message(d, fp, msg, 0);
+			libdax_msgs_submit(libdax_messenger,
+				d->global_index, 0x00000002,
+				LIBDAX_MSGS_SEV_DEBUG, LIBDAX_MSGS_PRIO_HIGH,
+				msg, 0, 0);
+			memset(c->page->data, 0, c->page->bytes);
+			c->error = 1;
+			c->dxfer_len = 0;
+		}
+#endif
+
 		/* ts A61010 */
 		/* a ssert(err != -1); */
 		if (err == -1) {
@@ -2184,7 +2215,9 @@ int sg_issue_command(struct burn_drive *d, struct command *c)
 			react_on_drive_loss(d, c, fp);
 			{ret = -1; goto ex;}
 		}
-                done = scsi_eval_cmd_outcome(d, c, fp, s.sbp, s.sb_len_wr,
+                done = scsi_eval_cmd_outcome(d, c, fp,
+		                             (unsigned char *) (s.sbp),
+		                             s.sb_len_wr,
 				             start_time, s.timeout, i, 0);
 		if (d->cancel)
 	break;
