@@ -1,7 +1,7 @@
 /* -*- indent-tabs-mode: t; tab-width: 8; c-basic-offset: 8; -*- */
 
 /* Copyright (c) 2004 - 2006 Derek Foreman, Ben Jansens
-   Copyright (c) 2006 - 2011 Thomas Schmitt <scdbackup@gmx.net>
+   Copyright (c) 2006 - 2014 Thomas Schmitt <scdbackup@gmx.net>
    Provided under GPL version 2 or later.
 */
 
@@ -65,19 +65,31 @@ struct command
 	int retry;
 	struct buffer *page;
 	int timeout; /* milliseconds */
+
+	double start_time;
+	double end_time;
+
 };
 
 struct burn_scsi_inquiry_data
 {
+	char peripheral; /* bit0-4: device type should be 5
+	                    bit5-7: qualifier must be 0 */
+
+	char version;    /* should be 3 (SPC-1) to 5 (SPC-3) (or higher ?)
+	                    but is often 0. */
+
 	char vendor[9];
 	char product[17];
 	char revision[5];
+
 	int valid;
 };
 
 
 struct scsi_mode_data
 {
+	int p2a_valid;
 	int buffer_size;
 	int dvdram_read;
 	int dvdram_write;
@@ -89,8 +101,13 @@ struct scsi_mode_data
 	int cdr_read;
 	int cdr_write;
 	int simulate;
+	int c2_pointers;
+	int underrun_proof;
+
 	int max_read_speed;
+	int cur_read_speed;
 	int max_write_speed;
+	int cur_write_speed;
 
 	/* ts A61021 */
 	int min_write_speed;
@@ -101,15 +118,10 @@ struct scsi_mode_data
 	int max_end_lba;
 	struct burn_speed_descriptor *speed_descriptors;
 
-	int cur_read_speed;
-	int cur_write_speed;
 	int retry_page_length;
 	int retry_page_valid;
 	int write_page_length;
 	int write_page_valid;
-	int c2_pointers;
-	int valid;
-	int underrun_proof;
 };
 
 
@@ -127,6 +139,26 @@ struct burn_format_descr {
 };
 
 
+/* ts B40106 : represents a Feature Descriptor as of mmc5r03c.pdf 5.2.2
+               There can be many of them. Thus a linked list.
+*/
+struct burn_feature_descr {
+	unsigned short feature_code;
+
+	unsigned char  flags; /* bit0= current
+                                 bit1= persistent
+                                 bit2-5= version
+                              */
+
+        unsigned char  data_lenght;
+
+	/* Additional bytes after the first 4 bytes of the descriptor */
+        unsigned char *data;
+
+        struct burn_feature_descr *next;
+};
+
+
 /** Gets initialized in enumerate_common() and burn_drive_register() */
 struct burn_drive
 {
@@ -178,12 +210,24 @@ struct burn_drive
 	unsigned char all_profiles[256];
 	int num_profiles;
 
+	/* ts B40106 : All feature descriptors as read from drive */
+	struct burn_feature_descr *features;
+
 	/* ts A70128 : MMC-to-MMC feature info from 46h for DVD-RW.
            Quite internal. Regard as opaque :)
 	*/
 	/* 1 = incremental recording available, 0 = not available */
 	int current_has_feat21h;
 
+	/* Some drives announce feature 21h on fast-blanked DVD-RW
+	   although they cannot write them in Incremental mode.
+	   0= does not look like the recent write run failed due to
+	      Incremental on fast blanked DVD-RW
+	   1= it seems to have happened
+	   2= it seems to have happened with write address 0
+	*/
+	int was_feat21h_failure;
+
 	/* Link Size item number 0 from feature 0021h descriptor */
 	int current_feat21h_link_size;
 
@@ -261,8 +305,18 @@ struct burn_drive
 	volatile int released;
 
 	/* ts A61106 */
+	/* 0= report errors
+	   1= do not report errors
+	   2= do not report errors which the libburn function indicates in
+	      member .had_particular_error
+	*/
 	int silent_on_scsi_error;
 
+	/* ts B21023 */
+	/* bit0= 5 64 00 occured with READ10 in mmc_read_10()
+	*/
+	int had_particular_error;
+
 	int stdio_fd;
 
 	int nwa;		/* next writeable address */
@@ -277,6 +331,12 @@ struct burn_drive
 	/* ts A90107 */
 	int state_of_last_session;
 
+#ifdef Libburn_disc_with_incomplete_sessioN
+	/* ts B30112 */
+	int incomplete_sessions;
+#endif
+
+
 	/* ts A70129 :
 	   from 51h READ DISC INFORMATION Last Track Number in Last Session */
 	int last_track_no;
@@ -336,6 +396,13 @@ struct burn_drive
 	volatile int cancel;
 	volatile enum burn_drive_status busy;
 
+	/* During write runs, this points to a copy of the applied
+	   struct burn_write_opts. Only read this underneath
+	   burn_disc_write_sync() which removes the copy when done.
+	   Especially do not read it from outside the write thread.
+	*/
+	struct burn_write_opts *write_opts;
+
 	/* ts A70929 */
 	pid_t thread_pid;
 	int thread_pid_valid;
@@ -372,10 +439,10 @@ struct burn_drive
 	int is_stopped;
 
 	void (*read_disc_info) (struct burn_drive *);
-	void (*read_sectors) (struct burn_drive *,
-			      int start,
-			      int len,
-			      const struct burn_read_opts *, struct buffer *);
+	int (*read_cd) (struct burn_drive *, int start, int len,
+	                int sec_type, int main_ch,
+	                const struct burn_read_opts *, struct buffer *,
+	                int flag);
 	void (*perform_opc) (struct burn_drive *);
 	void (*set_speed) (struct burn_drive *, int, int);
 	void (*send_parameters) (struct burn_drive *,
