@@ -1318,7 +1318,7 @@ int burn_disc_init_write_status(struct burn_write_opts *o,
 {
 	struct burn_drive *d = o->drive;
 	struct burn_track *t = NULL;
-	int sx, tx;
+	int sx, tx, ret;
 
 	d->cancel = 0;
 
@@ -1363,6 +1363,13 @@ int burn_disc_init_write_status(struct burn_write_opts *o,
 	if (o->fill_up_media && t != NULL)
 		burn_track_set_fillup(t, 1);
 
+	d->was_feat21h_failure = 0;
+	if(d->write_opts != NULL)
+		burn_write_opts_free(d->write_opts);
+	ret = burn_write_opts_clone(o, &(d->write_opts), 0);
+	if (ret <= 0)
+		return ret;
+
 	d->busy = BURN_DRIVE_WRITING;
 
 	return 1;
@@ -2124,13 +2131,16 @@ int burn_dvd_write_session(struct burn_write_opts *o,
 	if (d->current_profile == 0x11 || d->current_profile == 0x14 ||
 	    d->current_profile == 0x15) {
 		/* DVD-R , DVD-RW Sequential, DVD-R/DL Sequential */
-		multi_mem = o->multi;
-		if (!is_last_session)
-			o->multi = 1;
-		ret = burn_disc_close_session_dvd_minus_r(o);
-		o->multi = multi_mem;
-		if (ret <= 0)
-			return 0;
+		/* If feature 21h failed on write 0: do not close session */
+		if (d->was_feat21h_failure != 2) {
+			multi_mem = o->multi;
+			if (!is_last_session)
+				o->multi = 1;
+			ret = burn_disc_close_session_dvd_minus_r(o);
+			o->multi = multi_mem;
+			if (ret <= 0)
+				return 0;
+		}
 	} else if (d->current_profile == 0x12 || d->current_profile == 0x43) {
 		/* DVD-RAM , BD-RE */
 		/* ??? any finalization needed ? */;
@@ -2850,10 +2860,10 @@ ex:;
 		close(d->stdio_fd);
 	d->stdio_fd = -1;
 
-	/* update media state records */
+	/* update pseudo-media state records by re-grabbing */
 	burn_drive_mark_unready(d, 8);
+	burn_drive_grab_stdio(d, 1);
 
-	/* <<< d->busy = BURN_DRIVE_IDLE; */
 	return ret;
 }
 
@@ -3129,6 +3139,10 @@ ex:;
 		burn_os_free_buffer((char *) d->buffer,
 					sizeof(struct buffer), 0);
 	d->buffer = buffer_mem;
+	if (d->write_opts != NULL) {
+		burn_write_opts_free(d->write_opts);
+		d->write_opts = NULL;
+	}
 	return;
 }
 
@@ -3137,6 +3151,7 @@ int burn_random_access_write(struct burn_drive *d, off_t byte_address,
 				char *data, off_t data_count, int flag)
 {
 	int alignment = 0, start, upto, chunksize, err, fd = -1, ret;
+	int do_close = 0, getfl_ret;
 	char msg[81], *rpt;
 	struct buffer *buf = NULL, *buffer_mem = d->buffer;
 
@@ -3209,11 +3224,27 @@ int burn_random_access_write(struct burn_drive *d, off_t byte_address,
 			"Drive is busy on attempt to write random access",0,0);
 		{ret = 0; goto ex;}
 	}
-	if(d->drive_role != 1) {
-		fd = burn_stdio_open_write(d, byte_address, 2048, 0);
-		if (fd == -1)
-			{ret = 0; goto ex;}
+	if (d->drive_role != 1) {
+		if (d->stdio_fd >= 0) {
+			/* Avoid to have a read-only fd open */
+			getfl_ret = fcntl(d->stdio_fd, F_GETFL);
+			if (((O_RDWR | O_WRONLY | O_RDONLY) & getfl_ret) ==
+			    O_RDONLY) {
+				close(d->stdio_fd);
+				d->stdio_fd = -1;
+			}
+		}
+		if (d->stdio_fd >= 0) {
+			/* Avoid to have two fds open */
+			fd = d->stdio_fd;
+		} else {
+			fd = burn_stdio_open_write(d, byte_address, 2048, 0);
+			if (fd == -1)
+				{ret = 0; goto ex;}
+			do_close = 1;
+		}
 	}
+	d->cancel = 0;
 	d->busy = BURN_DRIVE_WRITING_SYNC;
 	d->buffer = buf;
 
@@ -3240,7 +3271,7 @@ int burn_random_access_write(struct burn_drive *d, off_t byte_address,
 		}
 		if (err == BE_CANCELLED) {
 			d->busy = BURN_DRIVE_IDLE;
-			if(fd >= 0)
+			if(fd >= 0 && do_close)
 				close(fd);
 			{ret = -(start * 2048 - byte_address); goto ex;}
 		}
@@ -3256,7 +3287,7 @@ int burn_random_access_write(struct burn_drive *d, off_t byte_address,
 		d->needs_sync_cache = 0;
 	}
 		
-	if(fd >= 0)
+	if(fd >= 0 && do_close)
 		close(fd);
 	d->buffer = buffer_mem;
 	d->busy = BURN_DRIVE_IDLE;
