@@ -1,6 +1,6 @@
 /*
  * Copyright (c) 2007 Vreixo Formoso
- * Copyright (c) 2009 Thomas Schmitt
+ * Copyright (c) 2009 - 2012 Thomas Schmitt
  *
  * This file is part of the libisofs project; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License version 2 
@@ -29,55 +29,73 @@
 #include <string.h>
 #include <stdio.h>
 
-static
-int get_iso_name(Ecma119Image *img, IsoNode *iso, char **name)
+/* @param flag   bit0=  Do not issue error messages
+*/
+int iso_get_ecma119_name(IsoWriteOpts *opts, char *input_charset, int imgid,
+                         char *node_name, enum IsoNodeType node_type,
+                         char **name, int flag)
 {
-    int ret, relaxed, free_ascii_name= 0, force_dots = 0, max_len;
+    int ret, relaxed, free_ascii_name = 0, force_dots = 0;
     char *ascii_name;
-    char *isoname= NULL;
+    char *isoname = NULL;
 
-    if (iso->name == NULL) {
+    if (node_name == NULL) {
         /* it is not necessarily an error, it can be the root */
         return ISO_SUCCESS;
     }
 
-    if (img->untranslated_name_len > 0) {
-        ascii_name = iso->name;
+    if (opts->untranslated_name_len > 0) {
+        ascii_name = node_name;
         ret = 1;
     } else {
-        ret = str2ascii(img->input_charset, iso->name, &ascii_name);
+        ret = str2ascii(input_charset, node_name, &ascii_name);
         free_ascii_name = 1;
     }
     if (ret < 0) {
-        iso_msg_submit(img->image->id, ret, 0,
-                       "Cannot convert name '%s' to ASCII", iso->name);
+        if (!(flag & 512))
+            iso_msg_submit(imgid, ret, 0,
+                           "Cannot convert name '%s' to ASCII", node_name);
         return ret;
     }
 
-    if (img->allow_full_ascii) {
+    if (opts->allow_full_ascii) {
         relaxed = 2;
     } else {
-        relaxed = (int)img->allow_lowercase;
+        relaxed = (int)opts->allow_lowercase;
     }
-    if (iso->type == LIBISO_DIR && !(img->allow_dir_id_ext)) {
-        if (img->untranslated_name_len > 0) {
-            if (strlen(ascii_name) > img->untranslated_name_len) {
+    if (opts->allow_7bit_ascii)
+        relaxed |= 4;
+    if (node_type == LIBISO_DIR && !(opts->allow_dir_id_ext)) {
+        if (opts->untranslated_name_len > 0) {
+            if (strlen(ascii_name) > opts->untranslated_name_len) {
 needs_transl:;
-                iso_msg_submit(img->image->id, ISO_NAME_NEEDS_TRANSL, 0,
+                if (!(flag & 512))
+                    iso_msg_submit(imgid, ISO_NAME_NEEDS_TRANSL, 0,
               "File name too long (%d > %d) for untranslated recording:  '%s'",
-                               strlen(ascii_name), img->untranslated_name_len,
-                               ascii_name);
+                          strlen(ascii_name), opts->untranslated_name_len,
+                          ascii_name);
                 return ISO_NAME_NEEDS_TRANSL;
             }
             isoname = strdup(ascii_name);
-        } else if (img->max_37_char_filenames) {
+        } else if (opts->max_37_char_filenames) {
             isoname = iso_r_dirid(ascii_name, 37, relaxed);
-        } else if (img->iso_level == 1) {
+        } else if (opts->iso_level == 1) {
+
+#ifdef Libisofs_old_ecma119_nameS
+
             if (relaxed) {
                 isoname = iso_r_dirid(ascii_name, 8, relaxed);
             } else {
-                isoname = iso_1_dirid(ascii_name);
+                isoname = iso_1_dirid(ascii_name, 0);
             }
+
+#else /* Libisofs_old_ecma119_nameS */
+
+            isoname = iso_1_dirid(ascii_name, relaxed);
+
+#endif /* ! Libisofs_old_ecma119_nameS */
+
+
         } else {
             if (relaxed) {
                 isoname = iso_r_dirid(ascii_name, 31, relaxed);
@@ -86,15 +104,21 @@ needs_transl:;
             }
         }
     } else {
-        force_dots = !((img->no_force_dots & 1) || iso->type == LIBISO_DIR);
-        if (img->untranslated_name_len > 0) {
-            if (strlen(ascii_name) > img->untranslated_name_len)
+        force_dots = !((opts->no_force_dots & 1) ||
+                       node_type == LIBISO_DIR);
+        if (opts->untranslated_name_len > 0) {
+            if (strlen(ascii_name) > opts->untranslated_name_len)
                 goto needs_transl;
             isoname = strdup(ascii_name);
-        } else if (img->max_37_char_filenames) {
+        } else if (opts->max_37_char_filenames) {
             isoname = iso_r_fileid(ascii_name, 36, relaxed, force_dots);
-        } else if (img->iso_level == 1) {
-            if (relaxed || !force_dots) {
+        } else if (opts->iso_level == 1) {
+
+#ifdef Libisofs_old_ecma119_nameS
+
+            int max_len;
+
+            if (relaxed) {
                 if (strchr(ascii_name, '.') == NULL)
                     max_len = 8;
                 else
@@ -102,8 +126,15 @@ needs_transl:;
                 isoname = iso_r_fileid(ascii_name, max_len, relaxed,
                                        force_dots);
             } else {
-                isoname = iso_1_fileid(ascii_name);
+                isoname = iso_1_fileid(ascii_name, 0, force_dots);
             }
+
+#else /* Libisofs_old_ecma119_nameS */
+
+            isoname = iso_1_fileid(ascii_name, relaxed, force_dots);
+
+#endif /* ! Libisofs_old_ecma119_nameS */
+
         } else {
             if (relaxed || !force_dots) {
                 isoname = iso_r_fileid(ascii_name, 30, relaxed, force_dots);
@@ -126,6 +157,25 @@ needs_transl:;
     }
 }
 
+static
+int get_iso_name(Ecma119Image *img, IsoNode *iso, char **name)
+{
+    int ret;
+
+    ret = iso_get_ecma119_name(img->opts, img->input_charset, img->image->id,
+                               iso->name, iso->type, name, 0);
+    return ret;
+}
+
+int ecma119_is_dedicated_reloc_dir(Ecma119Image *img, Ecma119Node *node)
+{
+    if (img->rr_reloc_node == node &&
+        node != img->root && node != img->partition_root &&
+        (img->opts->rr_reloc_flags & 2))
+        return 1;
+    return 0;
+}
+
 static
 int create_ecma119_node(Ecma119Image *img, IsoNode *iso, Ecma119Node **node)
 {
@@ -186,7 +236,7 @@ int create_file_src(Ecma119Image *img, IsoFile *iso, IsoFileSrc **src)
     off_t size;
 
     size = iso_stream_get_size(iso->stream);
-    if (size > (off_t)MAX_ISO_FILE_SECTION_SIZE && img->iso_level != 3) {
+    if (size > (off_t)MAX_ISO_FILE_SECTION_SIZE && img->opts->iso_level != 3) {
         char *ipath = iso_tree_get_node_path(ISO_NODE(iso));
         ret = iso_msg_submit(img->image->id, ISO_FILE_TOO_BIG, 0,
                               "File \"%s\" can't be added to image because "
@@ -327,6 +377,7 @@ int create_tree(Ecma119Image *image, IsoNode *iso, Ecma119Node **tree,
     int max_path;
     char *iso_name= NULL, *ipath = NULL;
     IsoFileSrc *src = NULL;
+    IsoWriteOpts *opts = image->opts;
 
     if (image == NULL || iso == NULL || tree == NULL) {
         return ISO_NULL_POINTER;
@@ -351,16 +402,16 @@ int create_tree(Ecma119Image *image, IsoNode *iso, Ecma119Node **tree,
             goto ex;
         }
         max_path = pathlen + 1 + (iso_name ? strlen(iso_name) : 0);
-        if (!image->rockridge) {
+        if (!opts->rockridge) {
             if ((iso->type == LIBISO_DIR && depth > 8) &&
-                !image->allow_deep_paths) {
-	        ipath = iso_tree_get_node_path(iso);
+                !opts->allow_deep_paths) {
+                ipath = iso_tree_get_node_path(iso);
                 ret = iso_msg_submit(image->image->id, ISO_FILE_IMGPATH_WRONG,
                                      0, "File \"%s\" can't be added, "
                                      "because directory depth "
                                      "is greater than 8.", ipath);
                 goto ex;
-            } else if (max_path > 255 && !image->allow_longer_paths) {
+            } else if (max_path > 255 && !opts->allow_longer_paths) {
                 ipath = iso_tree_get_node_path(iso);
                 ret = iso_msg_submit(image->image->id, ISO_FILE_IMGPATH_WRONG,
                                      0, "File \"%s\" can't be added, "
@@ -384,7 +435,7 @@ int create_tree(Ecma119Image *image, IsoNode *iso, Ecma119Node **tree,
             ret = 0; /* Hidden means non-existing */
             goto ex;
         }
-        if (image->rockridge) {
+        if (opts->rockridge) {
             ret = create_symlink(image, (IsoSymlink*)iso, &node);
         } else {
             /* symlinks are only supported when RR is enabled */
@@ -400,7 +451,7 @@ int create_tree(Ecma119Image *image, IsoNode *iso, Ecma119Node **tree,
             ret = 0; /* Hidden means non-existing */
             goto ex;
         }
-        if (image->rockridge) {
+        if (opts->rockridge) {
             ret = create_special(image, (IsoSpecial*)iso, &node);
         } else {
             /* special files are only supported when RR is enabled */
@@ -434,6 +485,15 @@ int create_tree(Ecma119Image *image, IsoNode *iso, Ecma119Node **tree,
                 if (ret < 0) {
                     goto ex;
                 }
+                if (depth == 1) { /* root is default */
+                    image->rr_reloc_node = node;
+                } else if (depth == 2) {
+                    /* Directories in root may be used as relocation dir */
+                    if (opts->rr_reloc_dir != NULL)
+                        if (opts->rr_reloc_dir[0] != 0 &&
+                            strcmp(iso->name, opts->rr_reloc_dir) == 0)
+                            image->rr_reloc_node = node;
+                }
             }
             ret = ISO_SUCCESS;
             pos = dir->children;
@@ -563,7 +623,7 @@ int mangle_single_dir(Ecma119Image *img, Ecma119Node *dir, int max_file_len,
             continue;
         }
 
-        if (img->untranslated_name_len) {
+        if (img->opts->untranslated_name_len) {
             /* This should not happen because no two IsoNode names should be
                identical and only unaltered IsoNode names should be seen here.
                Thus the Ema119Node names should be unique.
@@ -592,7 +652,8 @@ int mangle_single_dir(Ecma119Image *img, Ecma119Node *dir, int max_file_len,
             /* compute name and extension */
             dot = strrchr(full_name, '.');
             if (dot != NULL &&
-                (children[i]->type != ECMA119_DIR || img->allow_dir_id_ext)) {
+                (children[i]->type != ECMA119_DIR ||
+                 img->opts->allow_dir_id_ext)) {
 
                 /*
                  * File (normally not dir) with extension
@@ -678,8 +739,11 @@ int mangle_single_dir(Ecma119Image *img, Ecma119Node *dir, int max_file_len,
                         ret = ISO_OUT_OF_MEM;
                         goto mangle_cleanup;
                     }
+
+#ifdef Libisofs_extra_verbose_debuG
                     iso_msg_debug(img->image->id, "\"%s\" renamed to \"%s\"",
                                   children[k]->iso_name, new);
+#endif
 
                     iso_htable_remove_ptr(table, children[k]->iso_name, NULL);
                     free(children[k]->iso_name);
@@ -750,22 +814,24 @@ int mangle_dir(Ecma119Image *img, Ecma119Node *dir, int max_file_len,
 }
 
 static
-int mangle_tree(Ecma119Image *img, int recurse)
+int mangle_tree(Ecma119Image *img, Ecma119Node *dir, int recurse)
 {
     int max_file, max_dir;
     Ecma119Node *root;
 
-    if (img->untranslated_name_len > 0) {
-        max_file = max_dir = img->untranslated_name_len;
-    } else if (img->max_37_char_filenames) {
+    if (img->opts->untranslated_name_len > 0) {
+        max_file = max_dir = img->opts->untranslated_name_len;
+    } else if (img->opts->max_37_char_filenames) {
         max_file = max_dir = 37;
-    } else if (img->iso_level == 1) {
+    } else if (img->opts->iso_level == 1) {
         max_file = 12; /* 8 + 3 + 1 */
         max_dir = 8;
     } else {
         max_file = max_dir = 31;
     }
-    if (img->eff_partition_offset > 0) {
+    if (dir != NULL) {
+        root = dir;
+    } else if (img->eff_partition_offset > 0) {
         root = img->partition_root;
     } else {
         root = img->root;
@@ -887,41 +953,61 @@ int reparent(Ecma119Node *child, Ecma119Node *parent)
  *      1 success, < 0 error
  */
 static
-int reorder_tree(Ecma119Image *img, Ecma119Node *dir, int level, int pathlen)
+int reorder_tree(Ecma119Image *img, Ecma119Node *dir,
+                 int dir_level, int dir_pathlen)
 {
-    int ret;
-    size_t max_path;
-    Ecma119Node *root;
+    int ret, level, pathlen, newpathlen;
+    size_t max_path, i;
+    Ecma119Node *reloc, *child;
+
+    /* might change by relocation */
+    level = dir_level;
+    pathlen = dir_pathlen;
 
     max_path = pathlen + 1 + max_child_name_len(dir);
 
     if (level > 8 || max_path > 255) {
-        if (img->eff_partition_offset > 0) {
-            root = img->partition_root;
-        } else {
-            root = img->root;
+        reloc = img->rr_reloc_node;
+        if (reloc == NULL) {
+            if (img->eff_partition_offset > 0) {
+                reloc = img->partition_root;
+            } else {
+                reloc = img->root;
+            }
         }
-        ret = reparent(dir, root);
+        ret = reparent(dir, reloc);
         if (ret < 0) {
             return ret;
         }
 
-        /*
-         * we are appended to the root's children now, so there is no
-         * need to recurse (the root will hit us again)
-         */
-    } else {
-        size_t i;
+        if (reloc == img->root || reloc == img->partition_root) {
+            /*
+             * we are appended to the root's children now, so there is no
+             * need to recurse (the root will hit us again)
+             */
+            return ISO_SUCCESS;
+        }
 
-        for (i = 0; i < dir->info.dir->nchildren; i++) {
-            Ecma119Node *child = dir->info.dir->children[i];
-            if (child->type == ECMA119_DIR) {
-                int newpathlen = pathlen + 1 + strlen(child->iso_name);
-                ret = reorder_tree(img, child, level + 1, newpathlen);
-                if (ret < 0) {
-                    return ret;
-                }
-            }
+        /* dir is now the relocated Ecma119Node */
+        pathlen = 37 + 1; /* The dir name might get longer by mangling */
+        level = 2;
+        if (img->opts->rr_reloc_dir != NULL) {
+            pathlen += strlen(img->rr_reloc_node->iso_name) + 1;
+            if(img->opts->rr_reloc_dir[0] != 0)
+              level = 3;
+        }
+    }
+
+    if (ecma119_is_dedicated_reloc_dir(img, (Ecma119Node *) dir))
+        return ISO_SUCCESS;
+
+    for (i = 0; i < dir->info.dir->nchildren; i++) {
+        child = dir->info.dir->children[i];
+        if (child->type == ECMA119_DIR) {
+            newpathlen = pathlen + 1 + strlen(child->iso_name);
+            ret = reorder_tree(img, child, level + 1, newpathlen);
+            if (ret < 0)
+                return ret;
         }
     }
     return ISO_SUCCESS;
@@ -1060,7 +1146,7 @@ int match_hardlinks(Ecma119Image *img, Ecma119Node *dir, int flag)
         goto ex;
 
     /* Sort according to id tuples, IsoFileSrc identity, properties, xattr. */
-    if (img->hardlinks)
+    if (img->opts->hardlinks)
         qsort(nodes, node_count, sizeof(Ecma119Node *), ecma119_node_cmp_hard);
     else
         qsort(nodes, node_count, sizeof(Ecma119Node *),
@@ -1125,14 +1211,14 @@ int ecma119_tree_create(Ecma119Image *img)
     sort_tree(root);
 
     iso_msg_debug(img->image->id, "Mangling names...");
-    ret = mangle_tree(img, 1);
+    ret = mangle_tree(img, NULL, 1);
     if (ret < 0) {
         return ret;
     }
 
-    if (img->rockridge && !img->allow_deep_paths) {
+    if (img->opts->rockridge && !img->opts->allow_deep_paths) {
 
-        /* reorder the tree, acording to RRIP, 4.1.5 */
+        /* Relocate deep directories, acording to RRIP, 4.1.5 */
         ret = reorder_tree(img, root, 1, 0);
         if (ret < 0) {
             return ret;
@@ -1140,10 +1226,10 @@ int ecma119_tree_create(Ecma119Image *img)
 
         /*
          * and we need to remangle the root directory, as the function
-         * above could insert new directories into the root.
+         * above could insert new directories into the relocation directory.
          * Note that recurse = 0, as we don't need to recurse.
          */
-        ret = mangle_tree(img, 0);
+        ret = mangle_tree(img, img->rr_reloc_node, 0);
         if (ret < 0) {
             return ret;
         }
