@@ -1,6 +1,6 @@
 /*
  * Copyright (c) 2007 Vreixo Formoso
- * Copyright (c) 2009 - 2013 Thomas Schmitt
+ * Copyright (c) 2009 - 2014 Thomas Schmitt
  *
  * This file is part of the libisofs project; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License version 2 
@@ -85,6 +85,11 @@ int iso_image_new(const char *name, IsoImage **image)
     for (i = 0; i < 15; i++)
          img->mips_boot_file_paths[i] = NULL;
     img->sparc_core_node = NULL;
+    img->hppa_cmdline= NULL;
+    img->hppa_bootloader= NULL;
+    img->hppa_kernel_32= NULL;
+    img->hppa_kernel_64= NULL;
+    img->hppa_ramdisk= NULL;
     img->builder_ignore_acl = 1;
     img->builder_ignore_ea = 1;
     img->inode_counter = 0;
@@ -97,6 +102,7 @@ int iso_image_new(const char *name, IsoImage **image)
     img->generator_is_running = 0;
     for (i = 0; i < ISO_HFSPLUS_BLESS_MAX; i++)
         img->hfsplus_blessed[i] = NULL;
+    img->collision_warnings = 0;
 
     *image = img;
     return ISO_SUCCESS;
@@ -140,6 +146,13 @@ void iso_image_unref(IsoImage *image)
         iso_image_give_up_mips_boot(image, 0);
         if (image->sparc_core_node != NULL)
             iso_node_unref((IsoNode *) image->sparc_core_node);
+
+#ifdef Libisofs_enable_unreleased_hppa_palO
+
+        iso_image_set_hppa_palo(image, NULL, NULL, NULL, NULL, NULL, 1);
+
+#endif
+
         free(image->volset_id);
         free(image->volume_id);
         free(image->publisher_id);
@@ -350,13 +363,18 @@ int iso_image_set_pvd_times(IsoImage *image,
     if (creation_time == NULL || modification_time == NULL ||
         expiration_time == NULL || effective_time == NULL)
         return ISO_NULL_POINTER;
-    image->creation_time = strdup(creation_time);
-    image->modification_time = strdup(modification_time);
-    image->expiration_time = strdup(expiration_time);
-    image->effective_time = strdup(effective_time);
+    image->creation_time = calloc(18, 1);   /* Surely including a trailing 0 */
+    image->modification_time = calloc(18, 1);
+    image->expiration_time = calloc(18, 1);
+    image->effective_time = calloc(18, 1);
     if (image->creation_time == NULL || image->modification_time == NULL ||
         image->expiration_time == NULL || image->effective_time == NULL)
         return ISO_OUT_OF_MEM;
+    /* (If the string is too short, a non-zero timezone will not be stored) */
+    strncpy(image->creation_time, creation_time, 17);
+    strncpy(image->modification_time, modification_time, 17);
+    strncpy(image->expiration_time, expiration_time, 17);
+    strncpy(image->effective_time, effective_time, 17);
     return ISO_SUCCESS;
 }
 
@@ -858,3 +876,85 @@ int iso_image_get_sparc_core(IsoImage *img, IsoFile **sparc_core, int flag)
 }
 
 
+/* @param flag
+           bit0= Let NULL parameters free the corresponding image properties.
+                 Else only the non-NULL parameters of this call have an effect.
+*/
+static int hppa_palo_set_path(IsoImage *img, char *path, char **target,
+                              int flag)
+{
+    int ret;
+    IsoNode *node;
+    IsoFile *file;
+
+    if (path == NULL && !(flag & 1))
+        return ISO_SUCCESS;
+    if (iso_clone_mgtd_mem(path, target, 0) < 0)
+        return ISO_OUT_OF_MEM;
+    if (path == NULL)
+        return ISO_SUCCESS;
+    ret = iso_tree_path_to_node(img, path, &node);
+    if (ret < 0)
+        return ret;
+    if (ret == 0) {
+        iso_msg_submit(img->id, ISO_BOOT_FILE_MISSING, 0,
+                       "Cannot find in ISO image: HP-PA file '%s'", path);
+        return ISO_BOOT_FILE_MISSING;
+    }
+    if (iso_node_get_type(node) != LIBISO_FILE) {
+        iso_msg_submit(img->id, ISO_HPPA_PALO_NOTREG, 0,
+                       "HP-PA PALO file is not a data file: '%s'", path);
+        return ISO_HPPA_PALO_NOTREG;
+    }
+    file = (IsoFile *) node;
+    if (!file->explicit_weight)
+        file->sort_weight = 2;
+    return ISO_SUCCESS;
+}
+
+
+/* API */
+/* @param flag
+          Bitfield for control purposes
+           bit0= Let NULL parameters free the corresponding image properties.
+                 Else only the non-NULL parameters of this call have an effect.
+*/
+int iso_image_set_hppa_palo(IsoImage *img, char *cmdline, char *bootloader,
+                            char *kernel_32, char *kernel_64, char *ramdisk,
+                            int flag)
+{
+    int ret;
+
+    if (cmdline != NULL || (flag & 1))
+        if (iso_clone_mgtd_mem(cmdline, &(img->hppa_cmdline), 0) < 0)
+            return ISO_OUT_OF_MEM;
+    ret = hppa_palo_set_path(img, bootloader, &(img->hppa_bootloader),
+                             flag & 1);
+    if (ret < 0)
+        return ret;
+    ret = hppa_palo_set_path(img, kernel_32, &(img->hppa_kernel_32), flag & 1);
+    if (ret < 0)
+        return ret;
+    ret = hppa_palo_set_path(img, kernel_64, &(img->hppa_kernel_64), flag & 1);
+    if (ret < 0)
+        return ret;
+    ret = hppa_palo_set_path(img, ramdisk, &(img->hppa_ramdisk), flag & 1);
+    if (ret < 0)
+        return ret;
+    return ISO_SUCCESS;
+}
+
+
+/* API */
+int iso_image_get_hppa_palo(IsoImage *img, char **cmdline, char **bootloader,
+                            char **kernel_32, char **kernel_64, char **ramdisk)
+{
+    *cmdline = img->hppa_cmdline;
+    *bootloader = img->hppa_bootloader;
+    *kernel_32 = img->hppa_kernel_32;
+    *kernel_64 = img->hppa_kernel_64;
+    *ramdisk  = img->hppa_ramdisk;
+    return ISO_SUCCESS;
+}
+
+
