@@ -1,7 +1,7 @@
 
 /* xorriso - creates, loads, manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2013 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2014 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -211,6 +211,40 @@ int Xorriso_is_isohybrid(struct XorrisO *xorriso, IsoFile *bootimg_node,
 }
 
 
+/* @return <0 yes , 0 no , <0 error */
+int Xorriso_is_grub2_elto(struct XorrisO *xorriso, IsoFile *bootimg_node,
+                          int flag)
+{
+#define Xorriso_grub2_boot_info_poS 2548
+ int ret, i;
+ unsigned char buf[Xorriso_grub2_boot_info_poS + 8];
+ void *data_stream= NULL;
+ off_t blk;
+
+ ret= Xorriso_iso_file_open(xorriso, "", (void *) bootimg_node,
+                            &data_stream, 1);
+ if(ret <= 0)
+   return(-1);
+ ret= Xorriso_iso_file_read(xorriso, data_stream, (char *) buf,
+                            Xorriso_grub2_boot_info_poS  + 8, 0);
+ Xorriso_iso_file_close(xorriso, &data_stream, 0);
+ if(ret < Xorriso_grub2_boot_info_poS + 8)
+   return(0);
+
+ /* >>> ??? Check for some id */;
+
+ /* Check whether it has grub2_boot_info patching */
+ blk= 0;
+ for(i= 0; i < 8; i++)
+   blk|= buf[Xorriso_grub2_boot_info_poS + i] << (8 * i);
+ blk-= 5;
+ blk/= 4;
+ if(blk != (off_t) xorriso->loaded_boot_bin_lba)
+   return(0);
+ return(1);
+}
+
+
 int Xorriso_image_has_md5(struct XorrisO *xorriso, int flag)
 {
  int ret;
@@ -237,6 +271,27 @@ static const char *un0(const char *text)
 }
 
 
+static int Xorriso_report_pvd_time(struct XorrisO *xorriso, char *head,
+                                   char *pvd_time, int flag)
+{
+ char *msg, hr[17];
+ int at;
+
+ msg= xorriso->result_line;
+ strncpy(hr, pvd_time, 16);
+ hr[16]= 0;
+ sprintf(msg, "%s %s\n", head, hr);
+ Xorriso_result(xorriso,0);
+ if(pvd_time[16] != 0) {
+   at= abs(pvd_time[16]);
+   sprintf(msg, "%2.2s. Time Zone: %c%-2.2d:%-2.2d\n", head,
+           pvd_time[16] > 0 ? '+' : '-', at / 4, (at - (at / 4) * 4) * 15);
+   Xorriso_result(xorriso,0);
+ }
+ return(1);
+}
+
+
 int Xorriso_pvd_info(struct XorrisO *xorriso, int flag)
 {
  int ret, msc1= -1, msc2, i;
@@ -295,16 +350,11 @@ int Xorriso_pvd_info(struct XorrisO *xorriso, int flag)
 
  ret= iso_image_get_pvd_times(image, &crt, &mdt, &ext, &eft);
  if(ret != ISO_SUCCESS)
-   crt= mdt= ext= eft= "";
- sprintf(msg, "Creation Time: %s\n", crt);
- Xorriso_result(xorriso,0);
- sprintf(msg, "Modif. Time  : %s\n", mdt);
- Xorriso_result(xorriso,0);
- sprintf(msg, "Expir. Time  : %s\n", ext);
- Xorriso_result(xorriso,0);
- sprintf(msg, "Eff. Time    : %s\n", eft);
- Xorriso_result(xorriso,0);
-
+   crt= mdt= ext= eft= "                "; /* Need 17 bytes. Last byte 0. */
+ Xorriso_report_pvd_time(xorriso, "Creation Time:", crt, 0);
+ Xorriso_report_pvd_time(xorriso, "Modif. Time  :", mdt, 0);
+ Xorriso_report_pvd_time(xorriso, "Expir. Time  :", ext, 0);
+ Xorriso_report_pvd_time(xorriso, "Eff. Time    :", eft, 0);
  return(1);
 }
 
@@ -433,7 +483,14 @@ int Xorriso_boot_item_status(struct XorrisO *xorriso, char *cat_path,
 
  is_default= ((patch_isolinux & 1) == 0 || bin_path[0] == 0 || (flag & 4));
  sprintf(line, "-boot_image %s boot_info_table=%s\n",
-         (patch_isolinux & 2) ? "grub" : form, patch_isolinux ? "on" : "off");
+               (patch_isolinux & 2) ? "grub" : form,
+               (patch_isolinux & 1) ? "on" : "off");
+ if(!(is_default && no_defaults))
+   Xorriso_status_result(xorriso,filter,fp,flag&2); 
+ 
+ is_default= ((patch_isolinux & 512) == 0 || bin_path[0] == 0 || (flag & 4));
+ sprintf(line, "-boot_image grub grub2_boot_info=%s\n",
+               (patch_isolinux & 512) ? "on" : "off");
  if(!(is_default && no_defaults))
    Xorriso_status_result(xorriso,filter,fp,flag&2); 
  
@@ -474,6 +531,22 @@ ex:;
 }
 
 
+int Xorriso_status_hppa(struct XorrisO *xorriso, char *what, char *value,
+                        char *filter, FILE *fp, int flag)
+{
+ char *line;
+
+ line= xorriso->result_line;
+ if(value == NULL)
+   return(1);
+ sprintf(line, "-boot_image any hppa_%s=", what);
+ Text_shellsafe(value, line, 1);
+ strcat(line, "\n");
+ Xorriso_status_result(xorriso, filter, fp, flag & 2);
+ return(1);
+}
+
+
 /* 
  bit0= do only report non-default settings
  bit1= do only report to fp
@@ -484,6 +557,14 @@ int Xorriso_boot_status_non_mbr(struct XorrisO *xorriso, IsoImage *image,
  int i, num_boots, sa_type;
  char *paths[15], *line;
 
+#ifdef Libisofs_enable_unreleased_hppa_palO
+
+ int ret;
+ char num[4];
+ char *cmdline, *bootloader, *kernel_32, *kernel_64, *ramdisk;
+
+#endif
+
  line= xorriso->result_line;
 
  sa_type= (xorriso->system_area_options & 0xfc) >> 2;
@@ -492,23 +573,51 @@ int Xorriso_boot_status_non_mbr(struct XorrisO *xorriso, IsoImage *image,
    Text_shellsafe(xorriso->ascii_disc_label, line, 1);
    strcat(line, "\n");
    Xorriso_status_result(xorriso, filter, fp, flag & 2);
-   return(0);
- }
- if(sa_type != 1 && sa_type != 2)
-   return(0);
- num_boots= iso_image_get_mips_boot_files(image, paths, 0);
- Xorriso_process_msg_queues(xorriso, 0);
- if(num_boots <= 0)
-   return(num_boots);
- if(sa_type == 2)
-   num_boots= 1;
- for(i= 0; i < num_boots; i++) {
-   sprintf(line, "-boot_image any mips%s_path=", sa_type ==2 ? "el" : "");
-   Text_shellsafe(paths[i], line, 1);
+   sprintf(line, "-boot_image grub grub2_sparc_core=");
+   Text_shellsafe(xorriso->grub2_sparc_core, line, 1);
    strcat(line, "\n");
    Xorriso_status_result(xorriso, filter, fp, flag & 2);
+   return(0);
  }
- return(num_boots);
+ if(sa_type != 1 && sa_type != 2 && sa_type != 4 && sa_type != 5)
+   return(0);
+
+ if(sa_type == 1 || sa_type == 2) {
+   num_boots= iso_image_get_mips_boot_files(image, paths, 0);
+   Xorriso_process_msg_queues(xorriso, 0);
+   if(num_boots > 0) {
+     if(sa_type == 2)
+       num_boots= 1;
+     for(i= 0; i < num_boots; i++) {
+       sprintf(line, "-boot_image any mips%s_path=", sa_type ==2 ? "el" : "");
+       Text_shellsafe(paths[i], line, 1);
+       strcat(line, "\n");
+       Xorriso_status_result(xorriso, filter, fp, flag & 2);
+     }
+   }
+   return(num_boots);
+ }
+ if(sa_type == 4 || sa_type == 5) {
+
+#ifdef Libisofs_enable_unreleased_hppa_palO
+
+   ret= iso_image_get_hppa_palo(image, &cmdline, &bootloader, &kernel_32,
+                                &kernel_64, &ramdisk);
+   if(ret == 1) {
+     Xorriso_status_hppa(xorriso, "cmdline", cmdline, filter, fp, 0);
+     Xorriso_status_hppa(xorriso, "bootloader", bootloader, filter, fp, 0);
+     Xorriso_status_hppa(xorriso, "kernel_32", kernel_32, filter, fp, 0);
+     Xorriso_status_hppa(xorriso, "kernel_64", kernel_64, filter, fp, 0);
+     Xorriso_status_hppa(xorriso, "ramdisk", ramdisk, filter, fp, 0);
+     sprintf(num, "%d", sa_type);
+     Xorriso_status_hppa(xorriso, "hdrversion", num, filter, fp, 0);
+   }
+
+#endif /* ! Libisofs_enable_unreleased_hppa_palO */
+
+   return(0);
+ }
+ return(0);
 }
 
 
@@ -585,29 +694,26 @@ no_image:;
    }
    if(!(is_default && no_defaults))
       Xorriso_status_result(xorriso,filter,fp,flag&2); 
-   ret= 1; goto after_el_torito;
+   goto report_open_item;
  }
 
-
  if(xorriso->boot_image_bin_path[0] || xorriso->boot_count > 0)
    bin_path_in_use= 1;
- if(xorriso->boot_image_cat_path[0] && bin_path_in_use) {
-   is_default= 0;
-   sprintf(line,"-boot_image %s cat_path=", form);
-   Text_shellsafe(xorriso->boot_image_cat_path, line, 1);
-   strcat(line, "\n");
-   if(!(is_default && no_defaults))
-     Xorriso_status_result(xorriso,filter,fp,flag&2);
- }
- if(bin_path_in_use) {
-   is_default= !xorriso->boot_image_cat_hidden;
-   hpt= Xorriso__hide_mode_text(xorriso->boot_image_cat_hidden & 63, 0);
-   if(hpt != NULL)
-     sprintf(line, "-boot_image %s cat_hidden=%s\n", form, hpt);
-   Xorriso_free_meM(hpt);
-   if(!(is_default && no_defaults))
-     Xorriso_status_result(xorriso,filter,fp,flag&2);
- }
+
+ is_default= (xorriso->boot_image_cat_path[0] == 0);
+ sprintf(line,"-boot_image %s cat_path=", form);
+ Text_shellsafe(xorriso->boot_image_cat_path, line, 1);
+ strcat(line, "\n");
+ if(!(is_default && no_defaults))
+   Xorriso_status_result(xorriso,filter,fp,flag&2);
+
+ is_default= !xorriso->boot_image_cat_hidden;
+ hpt= Xorriso__hide_mode_text(xorriso->boot_image_cat_hidden & 63, 0);
+ if(hpt != NULL)
+   sprintf(line, "-boot_image %s cat_hidden=%s\n", form, hpt);
+ Xorriso_free_meM(hpt);
+ if(!(is_default && no_defaults))
+   Xorriso_status_result(xorriso,filter,fp,flag&2);
 
  if(xorriso->boot_count > 0) {
 
@@ -633,7 +739,7 @@ no_image:;
        else
          media_type= 0;
        ret= Xorriso_boot_item_status(xorriso, xorriso->boot_image_cat_path,
-                  path, platform_id, patch & 1, media_type,
+                  path, platform_id, patch, media_type,
                   load_size, id_string, sel_crit, "any",
                   filter, fp, 16 | (flag & 3));
        if(ret <= 0)
@@ -665,7 +771,8 @@ no_image:;
    if(i >= 28)
      {ret= 1; goto ex;} /* Images registered, pending is still default */
  }
-  
+
+report_open_item:;
  hflag= 16; 
  if(xorriso->boot_platform_id == 0xef && !xorriso->boot_efi_default)
    hflag= 0;
@@ -678,7 +785,6 @@ no_image:;
  if(ret <= 0)
    goto ex;
 
-after_el_torito:;
  ret = Xorriso_append_part_status(xorriso, image, filter, fp, flag & 3);
  if(ret <= 0)
    goto ex;
@@ -707,6 +813,8 @@ int Xorriso__append_boot_params(char *line, ElToritoBootImage *bootimg,
            " , platform_id=0x%-2.2X ", (unsigned int) platform_id);
  if(el_torito_seems_boot_info_table(bootimg, 0))
    sprintf(line + strlen(line), " , boot_info_table=on");
+ if(el_torito_seems_boot_info_table(bootimg, 1))
+   sprintf(line + strlen(line), " , grub2_boot_info=on");
  return(1);
 }
 
@@ -717,9 +825,10 @@ int Xorriso__append_boot_params(char *line, ElToritoBootImage *bootimg,
 */
 int Xorriso_show_boot_info(struct XorrisO *xorriso, int flag)
 {
- int ret, bin_path_valid= 0,has_isolinux_mbr= 0, i, num_boots;
+ int ret, bin_path_valid= 0,has_isolinux_mbr= 0, i, num_boots, has_mbr= 0;
+ int has_grub2_mbr= 0;
  unsigned int mbr_lba= 0;
- off_t lb0_count;
+ off_t lb0_count, blk;
  char *respt, *path;
  unsigned char *lb0= NULL;
  struct burn_drive_info *dinfo;
@@ -772,18 +881,19 @@ no_boot:;
      bin_path_valid= 1;
  }
  sprintf(respt, "Boot record  : El Torito");
+ /* Load and examine potential MBR */
+ ret= burn_read_data(drive, (off_t) 0, (char *) lb0, (off_t) 2048,
+                     &lb0_count, 2);
+ if(ret > 0)
+   if(lb0[510] == 0x55 && lb0[511] == 0xaa)
+     has_mbr= 1;
  if(bin_path_valid)
    ret= Xorriso_is_isohybrid(xorriso, bootimg_node, 0);
  else
    ret= 0;
  if(ret > 0) {
-   /* Load and examine potential MBR */
-   ret= burn_read_data(drive, (off_t) 0, (char *) lb0, (off_t) 2048,
-                       &lb0_count, 2);
-   if(ret > 0) {
+   if(has_mbr) {
      has_isolinux_mbr= 1;
-     if(lb0[510] != 0x55 || lb0[511] != 0xaa)
-       has_isolinux_mbr= 0;
      mbr_lba= lb0[432] | (lb0[433] << 8) | (lb0[434] << 16) | (lb0[435] << 24);
      mbr_lba/= 4;
      if(mbr_lba != (unsigned int) xorriso->loaded_boot_bin_lba)
@@ -795,17 +905,35 @@ no_boot:;
        if(i >= 426)
          has_isolinux_mbr= 0;
      }
-     for(i= 462; i < 510; i++)
-       if(lb0[i])
-     break;
-     if(i < 510)
-       has_isolinux_mbr= 0;
    }
    if(has_isolinux_mbr)
      strcat(respt, " , ISOLINUX isohybrid MBR pointing to boot image");
    else
      strcat(respt, " , ISOLINUX boot image capable of isohybrid");
  }
+ if(bin_path_valid)
+   ret= Xorriso_is_grub2_elto(xorriso, bootimg_node, 0);
+ else
+   ret= 0;
+ if(ret > 0) {
+   if(has_mbr) {
+     has_grub2_mbr= 1;
+     blk= 0;
+     for(i= 0; i < 8; i++)
+       blk|= lb0[i + 0x1b0] << (8 * i);
+     blk-= 4;
+     blk/= 4;
+     if(blk != (off_t) xorriso->loaded_boot_bin_lba)
+       has_grub2_mbr= 0;
+     for(i= 32; i < 427; i++)
+       if(strncmp((char *) (lb0 + i), "GRUB", 4) == 0)
+     break;
+     if(i >= 427)
+       has_grub2_mbr= 0;
+   }
+   if(has_grub2_mbr)
+     strcat(respt, " , GRUB2 MBR pointing to first boot image");
+ }
  strcat(respt, "\n");
  Xorriso_toc_line(xorriso, flag & 8);
  if(flag & 2)
@@ -1159,6 +1287,86 @@ report_error:;
 }
 
 
+/* @param flag bit0= Give up HP-PA boot parameters
+*/
+int Xorriso_set_hppa_boot_parm(struct XorrisO *xorriso, char *text, char *what,
+                               int flag)
+{
+ int ret;
+ IsoImage *image;
+ char *par[5];
+
+ ret= Xorriso_get_volume(xorriso, &image, 0);
+ if(ret <= 0)
+   return(ret);
+ par[0]= par[1]= par[2]= par[3]= par[4]= NULL;
+ if(flag & 1) {
+   /* Give up HP-PA boot parameters */
+
+#ifdef Libisofs_enable_unreleased_hppa_palO
+
+   iso_image_set_hppa_palo(image, par[0], par[1], par[2], par[3], par[4],
+                           1);
+
+#endif
+
+   return(1);
+ }
+ if(strcmp(what, "cmdline") == 0) {
+   par[0]= text;
+ } else if(strcmp(what, "bootloader") == 0) {
+   par[1]= text;
+ } else if(strcmp(what, "kernel_32") == 0 || strcmp(what, "kernel-32") == 0) {
+   par[2]= text;
+ } else if(strcmp(what, "kernel_64") == 0 || strcmp(what, "kernel-64") == 0) {
+   par[3]= text;
+ } else if(strcmp(what, "ramdisk") == 0) {
+   par[4]= text;
+ } else if(strcmp(what, "hdrversion") == 0) {
+   if(strcmp(text, "4") == 0) {
+     xorriso->system_area_options= (xorriso->system_area_options & ~0xfc) |
+                                   (4 << 2);
+   } else if(strcmp(text, "5") == 0) {
+     xorriso->system_area_options= (xorriso->system_area_options & ~0xfc) |
+                                   (5 << 2);
+   } else {
+     strcpy(xorriso->info_text, "Unsupported HP-PA PALO header version ");
+     Text_shellsafe(text, xorriso->info_text, 1);
+     Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+     return(0);
+   }
+   return(1);
+ } else {
+   strcpy(xorriso->info_text,
+          "HP-PA boot parameter name not recognized: hppa_");
+   Text_shellsafe(what, xorriso->info_text, 1);
+   Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+   return(0);
+ }
+
+#ifdef Libisofs_enable_unreleased_hppa_palO
+
+ ret= iso_image_set_hppa_palo(image, par[0], par[1], par[2], par[3], par[4],
+                              0);
+ if (ret < 0) {
+   Xorriso_report_iso_error(xorriso, "", ret,
+                            "Error when adding HP-PA boot parameter",
+                            0, "FAILURE", 1);
+   return(0);
+ }
+ return(1);
+
+#else
+
+ strcpy(xorriso->info_text, "HP-PA boot preparations are not yet supported");
+ Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+ return(0);
+
+#endif /* ! Libisofs_enable_unreleased_hppa_palO */
+
+}
+
+
 int Xorriso_coordinate_system_area(struct XorrisO *xorriso, int sa_type,
                                    int options, char *cmd, int flag)
 {
@@ -1168,7 +1376,7 @@ int Xorriso_coordinate_system_area(struct XorrisO *xorriso, int sa_type,
       "SUN Disk Label"};
 
  old_type= (xorriso->system_area_options & 0xfc) >> 2;
- old_options= xorriso->system_area_options & ~0xfc;
+ old_options= xorriso->system_area_options & ~0x40fc;
  if((old_type != 0 || old_options != 0) &&
     (old_type != sa_type || (old_options != 0 && old_options != options))) {
    sprintf(xorriso->info_text, "%s : First sector already occupied by %s",
@@ -1180,7 +1388,8 @@ int Xorriso_coordinate_system_area(struct XorrisO *xorriso, int sa_type,
    Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
    goto hint_revoke;
  }
- xorriso->system_area_options= ((sa_type << 2) & 0xfc) | (options & ~0xfc);
+ xorriso->system_area_options= (xorriso->system_area_options & 0x4000) |
+                               ((sa_type << 2) & 0xfc) | (options & ~0x40fc);
  if(sa_type == 0)
    xorriso->patch_system_area= xorriso->system_area_options;
  return(1);
