@@ -42,8 +42,8 @@
 /* @param eff_path  returns resulting effective path.
                     Must provide at least SfileadrL bytes of storage.
    @param flag bit0= do not produce problem events (unless faulty path format)
-               bit1= work purely literally, do not use libisofs
-               bit2= (with bit1) this is an address in the disk world
+               bit1= do not use libisofs (work literally or in disk world)
+               bit2= (implies bit1) this is an address in the disk world
                bit3= return root directory as "/" and not as ""
                bit4= (with bit2) determine type of disk file eff_path
                      and return 0 if not existing
@@ -59,10 +59,13 @@ int Xorriso_normalize_img_path(struct XorrisO *xorriso, char *wd,
  IsoImage *volume;
  IsoDir *dir= NULL;
  IsoNode *node= NULL;
- char *path= NULL, *apt, *npt, *cpt;
+ char *path= NULL, *apt, *npt, *cpt, *link_target= NULL;
 
  Xorriso_alloc_meM(path, char, SfileadrL);
+ Xorriso_alloc_meM(link_target, char, SfileadrL);
 
+ if(flag & 4)
+   flag|= 2;
  if((flag&64) || !(flag&2)) {
    ret= Xorriso_get_volume(xorriso, &volume, 0);
    if(ret<=0)
@@ -129,7 +132,24 @@ bonked_root:;
          {ret= -1; goto ex;}
        }
        dir= iso_node_get_parent(node);
+
+       /* >>> Get absolute path of node as eff_path, to avoid link problems */; 
+
      }
+     if(flag & 4) {
+       /* Linux-ish local filesystem semantics:
+          If the parent is a symbolic link, do not ascend to its parent,
+          but to the parent of the link target.
+          >>> ??? Are there systems which do it differently ?
+       */
+       if(Sfile_type(eff_path, 0) == 3) {
+         ret= Xorriso_resolve_link(xorriso, eff_path, link_target, 2);
+         if(ret <= 0)
+           {ret= -1; goto ex;}
+         strcpy(eff_path, link_target);
+       }
+     }
+
      /* truncate eff_path */;
      cpt= strrchr(eff_path, '/');
      if(cpt==NULL) /* ??? if not flag&2 then this is a bug */
@@ -171,6 +191,7 @@ much_too_long:;
  ret= 1+!!is_dir;
 ex:;
  Xorriso_free_meM(path);
+ Xorriso_free_meM(link_target);
  return(ret);
 }
 
@@ -705,6 +726,8 @@ int Xorriso_getfacl(struct XorrisO *xorriso, void *in_node, char *path,
 
  for(pass= 0; pass < 1 + (acl_text != NULL && !(flag & 2)); pass++) {
    if(pass) {
+     if(result_len == 0) 
+ break;
      *acl_text= calloc(result_len + 1, 1);
      if(*acl_text == NULL) {
        Xorriso_no_malloc_memory(xorriso, NULL, 0);
@@ -764,7 +787,13 @@ int Xorriso_getfacl(struct XorrisO *xorriso, void *in_node, char *path,
      }
    }
  }
- ret= 1;
+ if(result_len == 0) {
+   if(acl_text != NULL)
+     *acl_text= NULL;
+   ret= 2; 
+ } else {
+   ret= 1;
+ }
 ex:;
  iso_node_get_acl_text(node, &text, &d_text, 1 << 15);
  return(ret);
@@ -1448,9 +1477,12 @@ much_too_long:;
    report_size= sub_size/1024;
    if(report_size*1024<sub_size)
       report_size++;
-   sprintf(xorriso->result_line, "%7.f ",(double) (report_size));
+   if(xorriso->sh_style_result)
+     sprintf(xorriso->result_line, "%-7.f ",(double) (report_size));
+   else
+     sprintf(xorriso->result_line, "%7.f ",(double) (report_size));
    sprintf(xorriso->result_line+strlen(xorriso->result_line), "%s\n",
-           Text_shellsafe(show_path, sfe, 0));
+           Xorriso_esc_filepath(xorriso, show_path, sfe, 0));
    Xorriso_result(xorriso, 0);
  }
  ret= 1;
@@ -1621,7 +1653,7 @@ int Xorriso_ls_filev(struct XorrisO *xorriso, char *wd,
      if(filec>1) {
        strcpy(xorriso->result_line, "\n");
        Xorriso_result(xorriso,0);
-       Text_shellsafe(filev[i], xorriso->result_line, 0);
+       Xorriso_esc_filepath(xorriso, filev[i], xorriso->result_line, 0);
        strcat(xorriso->result_line, ":\n");
        Xorriso_result(xorriso,0);
      }
@@ -1670,14 +1702,17 @@ int Xorriso_ls_filev(struct XorrisO *xorriso, char *wd,
        if(ret==0)
  continue;
      }
-     sprintf(rpt, "%7.f ",(double) (size/1024));
+     if(xorriso->sh_style_result)
+       sprintf(rpt, "%-7.f ",(double) (size/1024));
+     else
+       sprintf(rpt, "%7.f ",(double) (size/1024));
    }
    if(link_target[0] && (flag&5)==1) {
-     Text_shellsafe(filev[i], xorriso->result_line, 1),
+     Xorriso_esc_filepath(xorriso, filev[i], xorriso->result_line, 1),
      strcat(xorriso->result_line, " -> ");
-     Text_shellsafe(link_target, xorriso->result_line, 1 | 2);
+     Xorriso_esc_filepath(xorriso, link_target, xorriso->result_line, 1 | 2);
    } else {
-     Text_shellsafe(filev[i], xorriso->result_line, 1);
+     Xorriso_esc_filepath(xorriso, filev[i], xorriso->result_line, 1);
    }
    strcat(xorriso->result_line, "\n");
    Xorriso_result(xorriso, 0);
@@ -1783,11 +1818,11 @@ cannot_create_iter:;
  continue;
    }
    if(link_target[0] && (flag&1)) {
-     Text_shellsafe(npt, xorriso->result_line, 1); 
+     Xorriso_esc_filepath(xorriso, npt, xorriso->result_line, 1); 
      strcat(xorriso->result_line, " -> ");
-     Text_shellsafe(link_target, xorriso->result_line, 1 | 2);
+     Xorriso_esc_filepath(xorriso, link_target, xorriso->result_line, 1 | 2);
    } else {
-     Text_shellsafe(npt, xorriso->result_line, 1);
+     Xorriso_esc_filepath(xorriso, npt, xorriso->result_line, 1);
    }
    strcat(xorriso->result_line, "\n");
    Xorriso_result(xorriso, 0);
@@ -1883,6 +1918,7 @@ ex:;
 
 /* @param flag bit0= a match count !=1 is a FAILURE event
                bit1= with bit0 tolerate 0 matches if pattern is a constant
+               bit2= do not issue debug messages about temporay memory needs
 */
 int Xorriso_expand_pattern(struct XorrisO *xorriso,
                            int num_patterns, char **patterns, int extra_filec,
@@ -1965,7 +2001,7 @@ int Xorriso_expand_pattern(struct XorrisO *xorriso,
  mem+= extra_filec*sizeof(char *);
  if(count<=0)
    {ret= 0; goto ex;}
- ret= Xorriso_alloc_pattern_mem(xorriso, *mem, count, filev, 0);
+ ret= Xorriso_alloc_pattern_mem(xorriso, *mem, count, filev, !!(flag & 4));
  if(ret<=0)
    goto ex;
  /* now store addresses */
