@@ -190,6 +190,8 @@ struct XorrisO { /* the global context of xorriso */
  int do_follow_param;
  int do_follow_links;
  int follow_link_limit;
+ int resolve_link_rec_count;
+ int resolve_link_rec_limit;
  int do_follow_mount;
  int do_global_uid;
  uid_t global_uid;
@@ -403,6 +405,10 @@ struct XorrisO { /* the global context of xorriso */
                                                1= MIPS Big Endian Volume Header
                                                2= MIPS Little Endian Boot Block
                                                3= SUN Disk Label for SUN SPARC
+                                               4= HP-PA PALO boot sector
+                                                  header version 4
+                                               5= HP-PA PALO boot sector
+                                                  header version 5
                                          bit8-9= Only with System area type 0
                                                Cylinder alignment mode
                                                0 = auto (align if bit1)
