@@ -4,7 +4,7 @@
    a command line oriented batch and dialog tool which creates, loads,
    manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2013 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2014 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -19,7 +19,7 @@
 */
 #define xorriso_libisoburn_req_major  1
 #define xorriso_libisoburn_req_minor  3
-#define xorriso_libisoburn_req_micro  4
+#define xorriso_libisoburn_req_micro  6
 
 
 struct SpotlisT;          /* List of intervals with different read qualities */
@@ -540,6 +540,9 @@ int Xorriso_boot_image_status(struct XorrisO *xorriso, char *filter, FILE *fp,
 
 int Xorriso_add_mips_boot_file(struct XorrisO *xorriso, char *path, int flag);
 
+int Xorriso_set_hppa_boot_parm(struct XorrisO *xorriso, char *text, char *what,
+                               int flag);
+
 int Xorriso_coordinate_system_area(struct XorrisO *xorriso, int sa_type,
                                    int options, char *cmd, int flag);
 
